@@ -42,7 +42,11 @@ namespace {
 
 	transaction_manager::ptr
 	open_tm(block_manager<>::ptr bm) {
-		space_map::ptr sm(new core_map(bm->get_nr_blocks()));
+		auto nr_blocks = bm->get_nr_blocks();
+		if (!nr_blocks)
+			throw runtime_error("Metadata is not large enough for superblock.");
+
+		space_map::ptr sm(new core_map(nr_blocks));
 		sm->inc(SUPERBLOCK_LOCATION);
 		transaction_manager::ptr tm(new transaction_manager(bm, sm));
 		return tm;
@@ -56,100 +60,10 @@ namespace {
 				lhs->set_count(b, rhs->get_count(b));
 		}
 	}
-
-	void print_superblock(superblock const &sb) {
-		using namespace std;
-
-		cerr << "superblock " << sb.csum_ << endl
-		     << "flags " << sb.flags_ << endl
-		     << "blocknr " << sb.blocknr_ << endl
-		     << "transaction id " << sb.trans_id_ << endl
-		     << "data mapping root " << sb.data_mapping_root_ << endl
-		     << "details root " << sb.device_details_root_ << endl
-		     << "data block size " << sb.data_block_size_ << endl
-		     << "metadata block size " << sb.metadata_block_size_ << endl
-		     << "metadata nr blocks " << sb.metadata_nr_blocks_ << endl
-		     << "metadata snapshot block " << sb.metadata_snap_ << endl
-			;
-	}
 }
 
 //----------------------------------------------------------------
 
-metadata::metadata(std::string const &dev_path, open_type ot,
-		   sector_t data_block_size, block_address nr_data_blocks)
-{
-	switch (ot) {
-	case OPEN:
-		tm_ = open_tm(open_bm(dev_path, block_io<>::READ_ONLY));
-		sb_ = read_superblock(tm_->get_bm());
-
-		if (sb_.version_ != 1)
-			throw runtime_error("unknown metadata version");
-
-		metadata_sm_ = open_metadata_sm(tm_, &sb_.metadata_space_map_root_);
-		tm_->set_sm(metadata_sm_);
-
-		data_sm_ = open_disk_sm(tm_, static_cast<void *>(&sb_.data_space_map_root_));
-
-		details_ = device_tree::ptr(
-			new device_tree(tm_, sb_.device_details_root_,
-					device_tree_detail::device_details_traits::ref_counter()));
-
-		mappings_top_level_ = dev_tree::ptr(
-			new dev_tree(tm_, sb_.data_mapping_root_,
-				     mapping_tree_detail::mtree_ref_counter(tm_)));
-
-		mappings_ = mapping_tree::ptr(
-			new mapping_tree(tm_, sb_.data_mapping_root_,
-					 mapping_tree_detail::block_time_ref_counter(data_sm_)));
-		break;
-
-	case CREATE:
-		tm_ = open_tm(open_bm(dev_path, block_io<>::READ_WRITE));
-		space_map::ptr core = tm_->get_sm();
-		metadata_sm_ = create_metadata_sm(tm_, tm_->get_bm()->get_nr_blocks());
-		copy_space_maps(metadata_sm_, core);
-		tm_->set_sm(metadata_sm_);
-
-		data_sm_ = create_disk_sm(tm_, nr_data_blocks);
-		details_ = device_tree::ptr(new device_tree(tm_, device_tree_detail::device_details_traits::ref_counter()));
-		mappings_ = mapping_tree::ptr(new mapping_tree(tm_,
-							       mapping_tree_detail::block_time_ref_counter(data_sm_)));
-		mappings_top_level_ = dev_tree::ptr(new dev_tree(tm_, mappings_->get_root(),
-								 mapping_tree_detail::mtree_ref_counter(tm_)));
-
-		::memset(&sb_, 0, sizeof(sb_));
-		sb_.magic_ = SUPERBLOCK_MAGIC;
-		sb_.version_ = 1;
-		sb_.data_mapping_root_ = mappings_->get_root();
-		sb_.device_details_root_ = details_->get_root();
-		sb_.data_block_size_ = data_block_size;
-		sb_.metadata_block_size_ = MD_BLOCK_SIZE;
-		sb_.metadata_nr_blocks_ = tm_->get_bm()->get_nr_blocks();
-
-		break;
-	}
-}
-
-metadata::metadata(std::string const &dev_path, block_address metadata_snap)
-{
-	tm_ = open_tm(open_bm(dev_path, block_io<>::READ_ONLY));
-	sb_ = read_superblock(tm_->get_bm(), metadata_snap);
-
-	// We don't open the metadata sm for a held root
-	//metadata_sm_ = open_metadata_sm(tm_, &sb_.metadata_space_map_root_);
-	//tm_->set_sm(metadata_sm_);
-
-	data_sm_ = open_disk_sm(tm_, static_cast<void *>(&sb_.data_space_map_root_));
-	details_ = device_tree::ptr(new device_tree(tm_, sb_.device_details_root_, device_tree_detail::device_details_traits::ref_counter()));
-	mappings_top_level_ = dev_tree::ptr(new dev_tree(tm_, sb_.data_mapping_root_,
-							 mapping_tree_detail::mtree_ref_counter(tm_)));
-	mappings_ = mapping_tree::ptr(new mapping_tree(tm_, sb_.data_mapping_root_,
-						       mapping_tree_detail::block_time_ref_counter(data_sm_)));
-}
-
-// FIXME: duplication
 metadata::metadata(block_manager<>::ptr bm, open_type ot,
 		   sector_t data_block_size,
 		   block_address nr_data_blocks)
@@ -162,29 +76,31 @@ metadata::metadata(block_manager<>::ptr bm, open_type ot,
 		if (sb_.version_ != 1)
 			throw runtime_error("unknown metadata version");
 
-		metadata_sm_ = open_metadata_sm(tm_, &sb_.metadata_space_map_root_);
+		metadata_sm_ = open_metadata_sm(*tm_, &sb_.metadata_space_map_root_);
 		tm_->set_sm(metadata_sm_);
 
-		data_sm_ = open_disk_sm(tm_, static_cast<void *>(&sb_.data_space_map_root_));
-		details_ = device_tree::ptr(new device_tree(tm_, sb_.device_details_root_, device_tree_detail::device_details_traits::ref_counter()));
-		mappings_top_level_ = dev_tree::ptr(new dev_tree(tm_, sb_.data_mapping_root_,
+		data_sm_ = open_disk_sm(*tm_, static_cast<void *>(&sb_.data_space_map_root_));
+		details_ = device_tree::ptr(new device_tree(*tm_, sb_.device_details_root_,
+							    device_tree_detail::device_details_traits::ref_counter()));
+		mappings_top_level_ = dev_tree::ptr(new dev_tree(*tm_, sb_.data_mapping_root_,
 								 mapping_tree_detail::mtree_ref_counter(tm_)));
-		mappings_ = mapping_tree::ptr(new mapping_tree(tm_, sb_.data_mapping_root_,
+		mappings_ = mapping_tree::ptr(new mapping_tree(*tm_, sb_.data_mapping_root_,
 							       mapping_tree_detail::block_time_ref_counter(data_sm_)));
 		break;
 
 	case CREATE:
 		tm_ = open_tm(bm);
 		space_map::ptr core = tm_->get_sm();
-		metadata_sm_ = create_metadata_sm(tm_, tm_->get_bm()->get_nr_blocks());
+		metadata_sm_ = create_metadata_sm(*tm_, tm_->get_bm()->get_nr_blocks());
 		copy_space_maps(metadata_sm_, core);
 		tm_->set_sm(metadata_sm_);
 
-		data_sm_ = create_disk_sm(tm_, nr_data_blocks);
-		details_ = device_tree::ptr(new device_tree(tm_, device_tree_detail::device_details_traits::ref_counter()));
-		mappings_ = mapping_tree::ptr(new mapping_tree(tm_,
+		data_sm_ = create_disk_sm(*tm_, nr_data_blocks);
+		details_ = device_tree::ptr(new device_tree(*tm_,
+							    device_tree_detail::device_details_traits::ref_counter()));
+		mappings_ = mapping_tree::ptr(new mapping_tree(*tm_,
 							       mapping_tree_detail::block_time_ref_counter(data_sm_)));
-		mappings_top_level_ = dev_tree::ptr(new dev_tree(tm_, mappings_->get_root(),
+		mappings_top_level_ = dev_tree::ptr(new dev_tree(*tm_, mappings_->get_root(),
 								 mapping_tree_detail::mtree_ref_counter(tm_)));
 
 		::memset(&sb_, 0, sizeof(sb_));
@@ -193,13 +109,43 @@ metadata::metadata(block_manager<>::ptr bm, open_type ot,
 		sb_.data_mapping_root_ = mappings_->get_root();
 		sb_.device_details_root_ = details_->get_root();
 		sb_.data_block_size_ = data_block_size;
-		sb_.metadata_block_size_ = MD_BLOCK_SIZE;
+		sb_.metadata_block_size_ = MD_BLOCK_SIZE >> SECTOR_SHIFT;
 		sb_.metadata_nr_blocks_ = tm_->get_bm()->get_nr_blocks();
 
 		break;
 	}
 }
 
+metadata::metadata(block_manager<>::ptr bm, bool read_space_maps)
+{
+	tm_ = open_tm(bm);
+	sb_ = read_superblock(tm_->get_bm(), SUPERBLOCK_LOCATION);
+
+	if (read_space_maps)
+		open_space_maps();
+
+	open_btrees();
+}
+
+metadata::metadata(block_manager<>::ptr bm,
+		   boost::optional<block_address> metadata_snap)
+{
+	tm_ = open_tm(bm);
+
+	superblock_detail::superblock actual_sb = read_superblock(bm, SUPERBLOCK_LOCATION);
+
+	if (!actual_sb.metadata_snap_)
+		throw runtime_error("no current metadata snap");
+
+	if (metadata_snap && *metadata_snap != actual_sb.metadata_snap_)
+			throw runtime_error("metadata snapshot does not match that in superblock");
+
+	sb_ = read_superblock(bm, actual_sb.metadata_snap_);
+
+	// metadata snaps don't record the space maps
+	open_btrees();
+}
+
 void
 metadata::commit()
 {
@@ -213,8 +159,26 @@ metadata::commit()
 	metadata_sm_->copy_root(&sb_.metadata_space_map_root_, sizeof(sb_.metadata_space_map_root_));
 
 	write_ref superblock = tm_->get_bm()->superblock_zero(SUPERBLOCK_LOCATION, superblock_validator());
-        superblock_disk *disk = reinterpret_cast<superblock_disk *>(superblock.data().raw());
+        superblock_disk *disk = reinterpret_cast<superblock_disk *>(superblock.data());
 	superblock_traits::pack(sb_, *disk);
 }
 
+void metadata::open_space_maps()
+{
+	metadata_sm_ = open_metadata_sm(*tm_, &sb_.metadata_space_map_root_);
+	tm_->set_sm(metadata_sm_);
+
+	data_sm_ = open_disk_sm(*tm_, static_cast<void *>(&sb_.data_space_map_root_));
+}
+
+void metadata::open_btrees()
+{
+	details_ = device_tree::ptr(new device_tree(*tm_, sb_.device_details_root_,
+						    device_tree_detail::device_details_traits::ref_counter()));
+	mappings_top_level_ = dev_tree::ptr(new dev_tree(*tm_, sb_.data_mapping_root_,
+							 mapping_tree_detail::mtree_ref_counter(tm_)));
+	mappings_ = mapping_tree::ptr(new mapping_tree(*tm_, sb_.data_mapping_root_,
+						       mapping_tree_detail::block_time_ref_counter(data_sm_)));
+}
+
 //----------------------------------------------------------------
